/*------------------------------------------------------------------------
 Module:        Tadam.h
 Author:        Oscar Garcia
 Creation Date: 24 April 2003
 Description:   TADAM dynamic link libraries (DLL) header file
------------------------------------------------------------------------*/

/*  TADAM is a stand growth model based on TASS, developed at the		*/
/*  University of Northern British Columbia.  More precisely, there     */
/*  are TADAM-df, TADAM-p, and TADAM-s, versions for Coastal			*/
/*  Douglas-fir, Interior lodgepole pine, and Interior white spruce,	*/
/*  respectively.														*/
/*  This is a common header file for the three DLL libraries			*/

/*  See:  http://www.unbc.ca/forestry/forestgrowth/tadam				*/


/*                  CONDITIONS												*/
/*																			*/
/* Copyright (c) 2002, 2003 Oscar Garcia									*/
/*																			*/
/* Permission is hereby granted, free of charge, to any person obtaining a	*/
/* copy of this software and associated documentation files (the			*/
/* "Software"), to use, copy, merge, publish, distribute, sublicense,		*/
/* and/or sell copies of the Software, and to permit persons to whom the	*/
/* Software is furnished to do so, subject to the following conditions:		*/
/* 																			*/
/* a) Modification of the Software is not allowed.							*/
/* 																			*/
/* b) The above copyright notice and this permission notice shall be		*/
/* included in all copies or substantial portions of the Software.			*/
/* 																			*/
/* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS	*/
/* OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF				*/
/* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.	*/
/* IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY		*/
/* CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,		*/
/* TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE		*/
/* SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.					*/



/*------------------------------------------------------------------------
 Procedure:     TadamVersion
 Purpose:       Return dll version number
 Input:         None
 Output:        Version number
 Errors:        None
------------------------------------------------------------------------*/
extern double TadamVersion(void);



/****  Primary growth functions  ****/


/*------------------------------------------------------------------------
 Procedure:     TadamGrowTph
 Purpose:       Project stems per hectare
 Note:			When possible, using TadamGrow is generally more
				efficient: Projecting tph and ba with TadamGrowTph and
				TadamGrowBa is equivalent to calling TadamGrow twice
 Input:         height  -  current site height (m)
                tph     -  current trees per hectare
                ba      -  current basal area (m2/ha)
                newht   -  final site height
 Output:        returns trees per hectare for newht
 Errors:        None
------------------------------------------------------------------------*/
extern double TadamGrowTph(double height, double tph, double ba, double newht);


/*------------------------------------------------------------------------
 Procedure:     TadamGrowBa
 Purpose:       Project basal area
 Note:			When possible, using TadamGrow is generally more
				efficient: Projecting tph and ba with TadamGrowTph and
				TadamGrowBa is equivalent to calling TadamGrow twice
 Input:         height  -  current site height (m)
                tph     -  current trees per hectare
                ba      -  current basal area (m2/ha)
                newht   -  final site height
 Output:        returns basal area for newht (m2/ha)
 Errors:        None
------------------------------------------------------------------------*/
extern double TadamGrowBa(double height, double tph, double ba, double newht);


/*------------------------------------------------------------------------
 Procedure:     TadamGrow
 Purpose:       Project stems per hectare and basal area
 Input:         height      -  current site height (m)
                tph     -  pointer to current trees per hectare
                ba      -  pointer to current basal area (m2/ha)
                newht   -  final site height
 Output:        tph     -  final trees per hectare updated in place
                ba      -  final basal area updated in place
 Errors:        None
------------------------------------------------------------------------*/
extern void TadamGrow(double height, double newht, double *tph, double *ba);



/****  Outputs and auxilliary relationships  ****/


/*------------------------------------------------------------------------
 Procedure:     TadamVolume
 Purpose:       Estimate volume per hectare
 Input:         height -  site height (m)
                tph    -  trees per hectare
                ba     -  basal area (m2/ha)
 Output:        returns total volume (m3/ha)
 Errors:        None
------------------------------------------------------------------------*/
extern double TadamVolume(double height, double tph, double ba);


/*------------------------------------------------------------------------
 Procedure:     TadamMerchVol
 Purpose:       Estimate volume per hectare for any merchantability limit
 Input:         height -  site height (m)
                tph    -  trees per hectare
                ba     -  basal area (m2/ha)
				dlim   -  limit dbh (cm)
 Output:        returns volume for trees above the limit dbh, excluding
 				a 10 cm top and 30 cm stump (m3/ha)
 Note:			under some conditions the estimated volume after thinning
				can be higher than that before thinning.  This is a
			    limitation of the model used
 Errors:        None
------------------------------------------------------------------------*/
extern double TadamMerchVol(double height, double tph, double ba, double dlim);


/*------------------------------------------------------------------------/*
 Procedure:     TadamBaAfter
 Purpose:       Estimate basal area after tinning
 Input:         height   -  site height (m)
                tph      -  trees per hectare before thinning
                ba       -  basal area before thinning (m2/ha)
                tphAfter -  trees per hectare after thinning
 Output:        returns basal area after thinning (m2/ha)
 Errors:        None
------------------------------------------------------------------------*/
extern double TadamBaAfter(double height, double tph, double ba, double tphAfter);


/*------------------------------------------------------------------------
 Procedure:     TadamTphAfter
 Purpose:       Estimate trees per hectare after tinning
 Input:         height  -  site height (m)
                tph     -  trees per hectare before thinning
                ba      -  basal area before thinning (m2/ha)
                baAfter -  basal area after thinning (m2/ha)
 Output:        returns trees per hectare after thinning
 Errors:        None
------------------------------------------------------------------------*/
extern double TadamTphAfter(double height, double tph, double ba, double baAfter);



/****  Site index - height - age calculations  ****/

/*  Sindex.dll (www.for.gov.bc.ca/research/software/sindex.htm)		*/
/*  should produce the same results, although the implementation	*/
/*  is different.  These functions specific to Tadam are			*/
/*  included here for convenience									*/


/*------------------------------------------------------------------------
 Procedure:     TadamSite
 Purpose:       Estimate site index from height and age
 Input:         height -  site height (m)
                age    -  age from planting (years)
                gain   -  time gain, negative of regen.delay (years)
 Output:        returns site index
 Errors:        returns 0 if calculation fails
------------------------------------------------------------------------*/
extern double TadamSite(double height, double age, double gain);


/*------------------------------------------------------------------------
 Procedure:     TadamGain
 Purpose:       Estimate time gain from site, height and age
 Input:         height -  site height (m)
                age    -  age from planting (years)
 				site   -  site index
 				Output: returns time gain (negative of regen. delay, years)
 Errors:        returns -999 if calculation fails
------------------------------------------------------------------------*/
extern double TadamGain(double height, double age, double site);


/*------------------------------------------------------------------------
 Procedure:     TadamHeight
 Purpose:       Estimate height from site index and age
 Input:         age  -  age from planting (years)
 				site -  site index
                gain -  time gain, negative of regen.delay (years)
 Output:        returns site height (m)
 Errors:        returns 0 if calculation fails
------------------------------------------------------------------------*/
extern double TadamHeight(double age, double site, double gain);


/*------------------------------------------------------------------------
 Procedure:     TadamAge
 Purpose:       Estimate age from site and height
 Input:         height -  site height (m)
 				site   -  site index
                gain   -  time gain, negative of regen. delay (years)
 Output:        returns age (years)
 Errors:        returns -999 if calculation fails
------------------------------------------------------------------------*/
extern double TadamAge(double height, double site, double gain);
